package com.lzmispt.data.resource.service.metadata.impl;

package .tmp;

import com.lzmispt.core.entity.dto.authorize.UserInfoDTO;
import com.lzmispt.data.center.entity.dto.databasestruct.DataSourceDTO;
import com.lzmispt.data.center.entity.enums.StatusCodeEnum;
import com.lzmispt.data.center.entity.vo.resource.datasource.DataSourceVO;
import com.lzmispt.data.center.entity.vo.standard.metadata.MetadataTypeVO;
import com.lzmispt.data.resource.dao.metadata.IMetadataDao;
import com.lzmispt.data.resource.entity.dto.resanddata.ResAndDataDirectoryDTO;
import com.lzmispt.data.resource.entity.po.metadata.MetadataDirectoryPO;
import com.lzmispt.data.resource.entity.po.metadata.MetadataEntityPO;
import com.lzmispt.data.resource.manager.metadata.MetadataManager;
import com.lzmispt.data.resource.service.metadata.impl.SpatialTableManager;
import com.lzmispt.dcbase.ServiceUtil;
import com.lzmispt.dcbase.databasestruct.DataStructProvider;
import com.lzmispt.dcbase.dcconst.DCConst;
import com.lzmispt.dcbase.util.SimpleCacheUtil;
import com.lzmispt.dcbase.util.TokenUtil;
import com.lzmispt.framework.entity.entity.ResultEntity;
import com.lzmispt.framework.exception.BusinessException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import org.springframework.beans.BeanUtils;

class ImportSpatialTableCache {
  private final Map<Long, DataSourceVO> dataSourceCache = new HashMap<>();
  
  private final Map<Long, MetadataDirectoryPO> directoryCache = new HashMap<>();
  
  private final Map<String, DataStructProvider> providerCache = new HashMap<>();
  
  private final Map<Integer, List<MetadataEntityPO>> entityListCache = new HashMap<>();
  
  private final Map<Long, Integer> nextChildOrderCache = new HashMap<>();
  
  private final String platCode;
  
  private final String tokenId;
  
  private final UserInfoDTO userInfo;
  
  private final Map<String, MetadataTypeVO> dataTypeMap;
  
  ImportSpatialTableCache(Map<String, MetadataTypeVO> dataTypeMap, String platCode, String tokenId) {
    this.platCode = platCode;
    this.tokenId = tokenId;
    this.userInfo = TokenUtil.getUserInfoByTokenId(tokenId);
    this.dataTypeMap = dataTypeMap;
  }
  
  public String getTokenId() {
    return this.tokenId;
  }
  
  public UserInfoDTO getUserInfo() {
    return this.userInfo;
  }
  
  public DataSourceVO getDataSource(Long id) {
    return (DataSourceVO)SimpleCacheUtil.cacheGet(this.dataSourceCache, id, () -> {
          DataSourceVO dataSourceVO = (DataSourceVO)SpatialTableManager.access$100().getDataSourceInfoDoNotDecryptById(id, this.platCode, this.tokenId).getObj();
          if (dataSourceVO == null)
            throw new BusinessException(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_GET_DB_INSTANCE_FAILED.getCode()), StatusCodeEnum.DATA_RESOURCE_GET_DB_INSTANCE_FAILED.getMessage()); 
          return dataSourceVO;
        });
  }
  
  public MetadataDirectoryPO getDirectory(Long id) {
    return (MetadataDirectoryPO)SimpleCacheUtil.cacheGet(this.directoryCache, id, () -> {
          ResAndDataDirectoryDTO resAndDataDirectoryDTO = MetadataManager.selectDirectoryByDirectoryId(id, this.platCode, DCConst.DataResourceCategory.DATA);
          if (resAndDataDirectoryDTO == null)
            throw new BusinessException(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_GET_PARENT_DIRECTORY_FAILED.getCode()), StatusCodeEnum.DATA_RESOURCE_GET_PARENT_DIRECTORY_FAILED.getMessage()); 
          MetadataDirectoryPO metadataDirectoryPO = new MetadataDirectoryPO();
          BeanUtils.copyProperties(resAndDataDirectoryDTO, metadataDirectoryPO);
          return (MetadataDirectoryPO)resAndDataDirectoryDTO;
        });
  }
  
  public DataStructProvider getProvider(Long id, String schema) {
    return (DataStructProvider)SimpleCacheUtil.cacheGet(this.providerCache, id + "_" + schema, () -> {
          DataSourceDTO dataSourceDTO = new DataSourceDTO();
          DataSourceVO dataSourcePO = getDataSource(id);
          BeanUtils.copyProperties(dataSourcePO, dataSourceDTO);
          if ("sqlserver".equals(dataSourceDTO.getType()))
            dataSourceDTO.setDbSchema(schema); 
          ResultEntity<DataStructProvider> dataBaseProvider = DataStructProvider.getDataBaseProvider(dataSourceDTO);
          if (!Objects.equals(dataBaseProvider.getCode(), Integer.valueOf(0)))
            throw new BusinessException(dataBaseProvider.getCode(), dataBaseProvider.getMsg()); 
          return (DataStructProvider)dataBaseProvider.getObj();
        });
  }
  
  public List<MetadataEntityPO> getEntityList(Integer dataCategory) {
    return (List<MetadataEntityPO>)SimpleCacheUtil.cacheGet(this.entityListCache, dataCategory, () -> ((IMetadataDao)ServiceUtil.getMetadataDao(this.platCode, IMetadataDao.class)).getAllMetadataEntities(dataCategory));
  }
  
  public int getNextChildOrder(MetadataDirectoryPO directory) {
    return ((Integer)this.nextChildOrderCache.compute(directory.getDirectoryId(), (k, v) -> {
          if (v == null) {
            MetadataDirectoryPO maxOrderDir = ((IMetadataDao)ServiceUtil.getMetadataDao(this.platCode, IMetadataDao.class)).selectMaxDirectoryOrderByDirectoryId(directory.getDirectoryId(), directory.getSpecialDatabaseId());
            return (maxOrderDir == null) ? Integer.valueOf(1) : Integer.valueOf(maxOrderDir.getDirectoryOrder().intValue() + 1);
          } 
          return Integer.valueOf(v.intValue() + 1);
        })).intValue();
  }
  
  public String getDBColumnType(String dbType, String dataType) {
    MetadataTypeVO metadataTypeVO = this.dataTypeMap.get(dataType);
    String dbColumnType = "";
    if (dbType.equalsIgnoreCase("mysql")) {
      dbColumnType = metadataTypeVO.getMysqlDbType();
    } else if (dbType.equalsIgnoreCase("sqlserver")) {
      dbColumnType = metadataTypeVO.getSqlDbType();
    } else if (dbType.equalsIgnoreCase("oracle")) {
      dbColumnType = metadataTypeVO.getOracleDbType();
    } else if (dbType.equalsIgnoreCase("postgresql")) {
      dbColumnType = metadataTypeVO.getPgsqlDbType();
    } else if (dbType.equalsIgnoreCase("dm")) {
      dbColumnType = metadataTypeVO.getDm8DbType();
    } else if (dbType.equalsIgnoreCase("kingbase")) {
      dbColumnType = metadataTypeVO.getKingBaseDbType();
    } 
    return dbColumnType;
  }
}
